#include <PSX/File/Processor/cd_file_processor.hpp>
#include <PSX/AutoLBA/auto_lba_declaration.hpp>
#include <PSX/GPU/gpu.hpp>
#include <PSX/GPU/gpu_primitives.hpp>
#include <stdio.h>

enum LBA {
    __jabyengine_start_lba_request
    __jabyengine_request_lba_for(FONT, "ASSETS/FONT.BIN"),
    __jabyengine_request_lba_for(ICON, "ASSETS/ICON.BIN"),
    __jabyengine_end_lba_request
};

static void load_assets() {
    static const JabyEngine::CDFile Assets[] = {
        JabyEngine::CDFileBuilder::simple_tim(LBA::FONT, JabyEngine::SimpleTIM(320,   0, 320, 510)),
        JabyEngine::CDFileBuilder::simple_tim(LBA::ICON, JabyEngine::SimpleTIM(320, 256, 320, 511)),
    };

    const auto                  buffer_cfg = JabyEngine::CDFileProcessor::BufferConfiguration::new_default();
    JabyEngine::CDFileProcessor file_processor;

    file_processor.setup(lba, Assets, buffer_cfg);
    while(true) {
        switch(file_processor.process()) {
            case JabyEngine::Progress::InProgress:
                break;

            case JabyEngine::Progress::Done:
                if(!file_processor.next(lba, buffer_cfg)) {
                    return;
                }
                break;
            
            case JabyEngine::Progress::Error:
                printf("Error detected! Aborting load\n");
                return;
        }
    }
    printf("Done loading assets!\n");
}

void main() {
    static constexpr auto TriangleColor = JabyEngine::GPU::Color24(0x0, 0xFF, 0xFF);
    static constexpr auto TriangleArea  = JabyEngine::GPU::AreaI16({0, 0}, {64, 64});
    static constexpr auto TriangleTPage = JabyEngine::GPU::TPage(320, 0, JabyEngine::GPU::SemiTransparency::B_Half_add_F_Half, JabyEngine::GPU::TexturePageColor::$4bit);
    static constexpr auto TriangleClut  = JabyEngine::GPU::PageClut(320, 510);

    static constexpr auto RectangleColor = JabyEngine::GPU::Color24(0x80, 0x80, 0xFF);
    static constexpr auto RectangleArea  = JabyEngine::GPU::AreaI16({0, TriangleArea.size.height}, {80, 80});
    static constexpr auto RectangleTPage = JabyEngine::GPU::TPage(320, 256, JabyEngine::GPU::SemiTransparency::B_Half_add_F_Half, JabyEngine::GPU::TexturePageColor::$4bit);
    static constexpr auto RectangleClut  = JabyEngine::GPU::PageClut(320, 511);

    const JabyEngine::GPU::POLY_F3  triangle1({
        {TriangleArea.position.x, TriangleArea.position.y},
        {TriangleArea.size.width, TriangleArea.size.height},
        {TriangleArea.position.x, TriangleArea.size.height}}, 
        TriangleColor
    );
    const JabyEngine::GPU::POLY_FT3 triangle2({
        {TriangleArea.position.x, TriangleArea.position.y}, 
        {TriangleArea.size.width, TriangleArea.position.y}, 
        {TriangleArea.size.width, TriangleArea.size.height}},{
        // Texture
        {TriangleArea.position.x, TriangleArea.position.y}, 
        {TriangleArea.size.width, TriangleArea.position.y}, 
        {TriangleArea.size.width, TriangleArea.size.height}},
        TriangleTPage,        
        TriangleClut,
        JabyEngine::GPU::Color24::White()
    );
    const JabyEngine::GPU::POLY_G3  triangle3({
        {triangle1.vertex0.move(TriangleArea.size.width, 0), JabyEngine::GPU::Color24::Red()},
        {triangle1.vertex1.move(TriangleArea.size.width, 0), JabyEngine::GPU::Color24::Green()},
        {triangle1.vertex2.move(TriangleArea.size.width, 0), JabyEngine::GPU::Color24::Blue()}}
    );
    const JabyEngine::GPU::POLY_GT3 triangle4({
        {triangle2.vertex0.move(TriangleArea.size.width, 0), triangle2.page0, JabyEngine::GPU::Color24::Red()},
        {triangle2.vertex1.move(TriangleArea.size.width, 0), triangle2.page1, JabyEngine::GPU::Color24::Blue()},
        {triangle2.vertex2.move(TriangleArea.size.width, 0), triangle2.page2, JabyEngine::GPU::Color24::Green()}},
        TriangleTPage,
        TriangleClut
    );

    const JabyEngine::GPU::POLY_F4 rectangle(RectangleArea, RectangleColor);

    load_assets();

    while(true) {
        JabyEngine::GPU::render(triangle1);
        JabyEngine::GPU::render(triangle2);
        JabyEngine::GPU::render(triangle3);
        JabyEngine::GPU::render(triangle4);

        JabyEngine::GPU::render(rectangle);

        JabyEngine::GPU::swap_buffers_vsync(2);
    }
}
__declare_lba_header(LBA);